@@ -30,7 +30,7 @@ def variance_from_spectra(
         Power/cross spectra for each channel.
         Can be an (n_channels, n_channels) array or (n_channels,) array.
     components : np.ndarray
-        Spectral components. Shape is (n_components, n_f).
+        Spectral components. Shape is (n_components, n_freq).
     frequency_range : list
         Frequency range to integrate the PSD over (Hz). Default is full range.
 
@@ -46,13 +46,13 @@ def variance_from_spectra(
     if power_spectra.ndim == 2:
         # PSDs were passed
         power_spectra = power_spectra[np.newaxis, np.newaxis, ...]
-        n_subjects, n_modes, n_channels, n_f = power_spectra.shape
+        n_subjects, n_modes, n_channels, n_freq = power_spectra.shape
 
     elif power_spectra.shape[-2] != power_spectra.shape[-3]:
         # PSDs were passed, check dimensionality
         error_message = (
-            "A (n_channels, n_f), (n_modes, n_channels, n_f) or "
-            + "(n_subjects, n_modes, n_channels, n_f) array must be passed."
+            "A (n_channels, n_freq), (n_modes, n_channels, n_freq) or "
+            + "(n_subjects, n_modes, n_channels, n_freq) array must be passed."
         )
         power_spectra = array_ops.validate(
             power_spectra,
@@ -60,14 +60,14 @@ def variance_from_spectra(
             allow_dimensions=[2, 3],
             error_message=error_message,
         )
-        n_subjects, n_modes, n_channels, n_f = power_spectra.shape
+        n_subjects, n_modes, n_channels, n_freq = power_spectra.shape
 
     else:
         # Cross spectra were passed, check dimensionality
         error_message = (
-            "A (n_channels, n_channels, n_f), "
-            + "(n_modes, n_channels, n_channels, n_f) or "
-            + "(n_subjects, n_modes, n_channels, n_channels, n_f) "
+            "A (n_channels, n_channels, n_freq), "
+            + "(n_modes, n_channels, n_channels, n_freq) or "
+            + "(n_subjects, n_modes, n_channels, n_channels, n_freq) "
             + "array must be passed."
         )
         power_spectra = array_ops.validate(
@@ -76,7 +76,7 @@ def variance_from_spectra(
             allow_dimensions=[3, 4],
             error_message=error_message,
         )
-        n_subjects, n_modes, n_channels, n_channels, n_f = power_spectra.shape
+        n_subjects, n_modes, n_channels, n_channels, n_freq = power_spectra.shape
 
     if components is not None and frequency_range is not None:
         raise ValueError(
@@ -108,7 +108,7 @@ def variance_from_spectra(
             psd = power_spectra[i]
 
         # Concatenate over modes
-        psd = psd.reshape(-1, n_f)
+        psd = psd.reshape(-1, n_freq)
         psd = psd.real
 
         if components is not None:
@@ -182,9 +182,9 @@ def power_map_grid(mask_file, parcellation_file, power_map):
 
 def save(
     power_map,
-    filename,
     mask_file,
     parcellation_file,
+    filename=None,
     component=0,
     subtract_mean=False,
     mean_weights=None,
@@ -199,13 +199,14 @@ def save(
         (n_modes, n_channels) or (n_channels,). A (..., n_channels, n_channels)
         array can also be passed. Warning: this function cannot be used if n_modes
         is equal to n_channels.
-    filename : str
-        Output filename. If extension is .nii.gz the power map is saved as a
-        NIFTI file. Or if the extension is png/svg/pdf, it is saved as images.
     mask_file : str
         Mask file used to preprocess the training data.
     parcellation_file : str
         Parcellation file used to parcelate the training data.
+    filename : str
+        Output filename. If extension is .nii.gz the power map is saved as a
+        NIFTI file. Or if the extension is png/svg/pdf, it is saved as images.
+        Optional, if None is passed then the image is shown on screen.
     component : int
         Spectral component to save.
     subtract_mean : bool
@@ -220,12 +221,13 @@ def save(
     power_map = np.copy(power_map)
 
     # Validation
-    allowed_extensions = [".nii.gz", ".png", ".svg", ".pdf"]
-    if not any([ext in filename for ext in allowed_extensions]):
-        raise ValueError(
-            "filename must have one of following extensions: "
-            + f"{' '.join(allowed_extensions)}."
-        )
+    if filename is not None:
+        allowed_extensions = [".nii", ".nii.gz", ".png", ".svg", ".pdf"]
+        if not any([ext in filename for ext in allowed_extensions]):
+            raise ValueError(
+                "filename must have one of following extensions: "
+                + f"{' '.join(allowed_extensions)}."
+            )
     mask_file = files.check_exists(mask_file, files.mask.directory)
     parcellation_file = files.check_exists(
         parcellation_file, files.parcellation.directory
@@ -271,36 +273,51 @@ def save(
     # Load the mask
     mask = nib.load(mask_file)
 
-    # Save as nii file
-    if ".nii.gz" in filename:
-        print(f"Saving {filename}")
-        nii = nib.Nifti1Image(power_map, mask.affine, mask.header)
-        nib.save(nii, filename)
+    # Number of modes
+    n_modes = power_map.shape[-1]
 
-    # Save each map as an image
-    else:
-        n_modes = power_map.shape[-1]
+    # Just display the power map
+    if filename is None:
         for i in trange(n_modes, desc="Saving images", ncols=98):
             nii = nib.Nifti1Image(power_map[:, :, :, i], mask.affine, mask.header)
-            output_file = "{fn.parent}/{fn.stem}{i:0{w}d}{fn.suffix}".format(
-                fn=Path(filename), i=i, w=len(str(n_modes))
-            )
             plotting.plot_img_on_surf(
                 nii,
                 views=["lateral", "medial"],
                 hemispheres=["left", "right"],
                 colorbar=True,
-                output_file=output_file,
                 **plot_kwargs,
             )
 
+    else:
+        # Save as nii file
+        if ".nii" in filename:
+            print(f"Saving {filename}")
+            nii = nib.Nifti1Image(power_map, mask.affine, mask.header)
+            nib.save(nii, filename)
+
+        else:
+            # Save each map as an image
+            for i in trange(n_modes, desc="Saving images", ncols=98):
+                nii = nib.Nifti1Image(power_map[:, :, :, i], mask.affine, mask.header)
+                output_file = "{fn.parent}/{fn.stem}{i:0{w}d}{fn.suffix}".format(
+                    fn=Path(filename), i=i, w=len(str(n_modes))
+                )
+                plotting.plot_img_on_surf(
+                    nii,
+                    views=["lateral", "medial"],
+                    hemispheres=["left", "right"],
+                    colorbar=True,
+                    output_file=output_file,
+                    **plot_kwargs,
+                )
+
 
 def multi_save(
     group_power_map,
     subject_power_map,
-    filename,
     mask_file,
     parcellation_file,
+    filename=None,
     subjects=None,
     component=0,
     subtract_mean=False,
@@ -325,13 +342,14 @@ def multi_save(
         (n_subjects, n_modes, n_channels), (n_modes, n_channels) or (n_channels,).
         A (..., n_channels, n_channels) array can also be passed.
         Warning: this function cannot be used if n_modes = n_channels.
-    filename : str
-        Output filename. If extension is .nii.gz the power map is saved as a
-        NIFTI file. Or if the extension is png, it is saved as images.
     mask_file : str
         Mask file used to preprocess the training data.
     parcellation_file : str
         Parcellation file used to parcelate the training data.
+    filename : str
+        Output filename. If extension is .nii.gz the power map is saved as a
+        NIFTI file. Or if the extension is png, it is saved as images.
+        Optional, if None is passed then the image is shown on screen.
     subjects : list
         List of subject indices to be plot power maps for.
     component : int
