@@ -3,6 +3,6 @@
 """
 
 from osl_dynamics.data.base import Data
-from osl_dynamics.data.osl import OSL_HMM
+from osl_dynamics.data.osl import OSL_HMM, HMM_MAR
 
-__all__ = ["Data", "OSL_HMM"]
+__all__ = ["Data", "OSL_HMM", "HMM_MAR"]
