@@ -15,8 +15,9 @@ from osl_dynamics.inference.initializers import WeightInitializer
 from osl_dynamics.inference.layers import (
     add_epsilon,
     LogLikelihoodLossLayer,
-    MeanVectorsLayer,
+    VectorsLayer,
     CovarianceMatricesLayer,
+    DiagonalMatricesLayer,
     MixVectorsLayer,
     MixMatricesLayer,
 )
@@ -44,7 +45,10 @@ class Config(BaseModelConfig):
     initial_means : np.ndarray
         Initialisation for mean vectors.
     initial_covariances : np.ndarray
-        Initialisation for mode covariances.
+        Initialisation for state covariances. If diagonal_covariances=True
+        and full matrices are passed, the diagonal is extracted.
+    diagonal_covariances : bool
+        Should we learn diagonal mode covariances?
     covariances_epsilon : float
         Error added to mode covariances for numerical stability.
     means_regularizer : tf.keras.regularizers.Regularizer
@@ -76,6 +80,7 @@ class Config(BaseModelConfig):
     learn_covariances: bool = None
     initial_means: np.ndarray = None
     initial_covariances: np.ndarray = None
+    diagonal_covariances: bool = False
     covariances_epsilon: float = None
     means_regularizer: tf.keras.regularizers.Regularizer = None
     covariances_regularizer: tf.keras.regularizers.Regularizer = None
@@ -118,7 +123,7 @@ class Model(ModelBase):
         covariances : np.ndarary
             Mode covariances.
         """
-        return get_covariances(self.model)
+        return get_covariances(self.model, self.config.diagonal_covariances)
 
     def get_means_covariances(self):
         """Get the means and covariances of each mode.
@@ -130,7 +135,7 @@ class Model(ModelBase):
         covariances : np.ndarray
             Mode covariances.
         """
-        return get_means_covariances(self.model)
+        return get_means_covariances(self.model, self.config.diagonal_covariances)
 
     def set_means(self, means, update_initializer=True):
         """Set the means of each mode.
@@ -156,14 +161,21 @@ class Model(ModelBase):
             Do we want to use the passed covariances when we re-initialize
             the model?
         """
-        set_covariances(self.model, covariances, update_initializer)
+        set_covariances(
+            self.model,
+            covariances,
+            self.config.diagonal_covariances,
+            update_initializer,
+        )
 
     def set_regularizers(self, training_dataset):
         """Set the means and covariances regularizer based on the training data.
 
         A multivariate normal prior is applied to the mean vectors with mu = 0,
-        sigma=diag((range / 2)**2) and an inverse Wishart prior is applied to the
-        covariances matrices with nu=n_channels - 1 + 0.1 and psi=diag(range).
+        sigma=diag((range / 2)**2). If config.diagonal_covariances is True, a log
+        normal prior is applied to the diagonal of the covariances matrices with mu=0,
+        sigma=sqrt(log(2 * (range))), otherwise an inverse Wishart prior is applied
+        to the covariances matrices with nu=n_channels - 1 + 0.1 and psi=diag(1 / range).
 
         Parameters
         ----------
@@ -174,7 +186,9 @@ class Model(ModelBase):
             set_means_regularizer(self.model, training_dataset)
 
         if self.config.learn_covariances:
-            set_covariances_regularizer(self.model, training_dataset)
+            set_covariances_regularizer(
+                self.model, training_dataset, self.config.diagonal_covariances
+            )
 
 
 def _model_structure(config):
@@ -190,7 +204,7 @@ def _model_structure(config):
     #   and the observation model.
 
     # Definition of layers
-    means_layer = MeanVectorsLayer(
+    means_layer = VectorsLayer(
         config.n_modes,
         config.n_channels,
         config.learn_means,
@@ -198,15 +212,26 @@ def _model_structure(config):
         config.means_regularizer,
         name="means",
     )
-    covs_layer = CovarianceMatricesLayer(
-        config.n_modes,
-        config.n_channels,
-        config.learn_covariances,
-        config.initial_covariances,
-        config.covariances_epsilon,
-        config.covariances_regularizer,
-        name="covs",
-    )
+    if config.diagonal_covariances:
+        covs_layer = DiagonalMatricesLayer(
+            config.n_modes,
+            config.n_channels,
+            config.learn_covariances,
+            config.initial_covariances,
+            config.covariances_epsilon,
+            config.covariances_regularizer,
+            name="covs",
+        )
+    else:
+        covs_layer = CovarianceMatricesLayer(
+            config.n_modes,
+            config.n_channels,
+            config.learn_covariances,
+            config.initial_covariances,
+            config.covariances_epsilon,
+            config.covariances_regularizer,
+            name="covs",
+        )
     mix_means_layer = MixVectorsLayer(name="mix_means")
     mix_covs_layer = MixMatricesLayer(name="mix_covs")
     ll_loss_layer = LogLikelihoodLossLayer(config.covariances_epsilon, name="ll_loss")
@@ -221,23 +246,33 @@ def _model_structure(config):
     return tf.keras.Model(inputs=[data, alpha], outputs=[ll_loss], name="DyNeMo-Obs")
 
 
-def get_covariances(model):
+def get_means(model):
+    means_layer = model.get_layer("means")
+    means = means_layer.vectors
+    return means.numpy()
+
+
+def get_covariances(model, diagonal=False):
     covs_layer = model.get_layer("covs")
-    covs = add_epsilon(
-        covs_layer.bijector(covs_layer.flattened_cholesky_factors), covs_layer.epsilon
-    )
+    if diagonal:
+        covs = add_epsilon(
+            tf.linalg.diag(covs_layer.bijector(covs_layer.diagonals)),
+            covs_layer.epsilon,
+            diag=True,
+        )
+    else:
+        covs = add_epsilon(
+            covs_layer.bijector(covs_layer.flattened_cholesky_factors),
+            covs_layer.epsilon,
+            diag=True,
+        )
     return covs.numpy()
 
 
-def get_means_covariances(model):
-    means_layer = model.get_layer("means")
-    covs_layer = model.get_layer("covs")
-
-    means = means_layer.vectors
-    covs = add_epsilon(
-        covs_layer.bijector(covs_layer.flattened_cholesky_factors), covs_layer.epsilon
-    )
-    return means.numpy(), covs.numpy()
+def get_means_covariances(model, diagonal=False):
+    means = get_means(model)
+    covs = get_covariances(model, diagonal)
+    return means, covs
 
 
 def set_means(model, means, update_initializer=True, layer_name="means"):
@@ -250,17 +285,32 @@ def set_means(model, means, update_initializer=True, layer_name="means"):
         means_layer.vectors_initializer = WeightInitializer(means)
 
 
-def set_covariances(model, covariances, update_initializer=True, layer_name="covs"):
+def set_covariances(
+    model, covariances, diagonal=False, update_initializer=True, layer_name="covs"
+):
     covariances = covariances.astype(np.float32)
     covs_layer = model.get_layer(layer_name)
-    layer_weights = covs_layer.flattened_cholesky_factors
-    flattened_cholesky_factors = covs_layer.bijector.inverse(covariances)
-    layer_weights.assign(flattened_cholesky_factors)
 
-    if update_initializer:
-        covs_layer.flattened_cholesky_factors_initializer = WeightInitializer(
-            flattened_cholesky_factors
-        )
+    if diagonal:
+        if covariances.ndim == 3:
+            # Only keep the diagonal as a vector
+            covariances = np.diagonal(covariances, axis1=1, axis2=2)
+        diagonals = covs_layer.bijector.inverse(covariances)
+        layer_weights = covs_layer.diagonals
+        layer_weights.assign(diagonals)
+
+        if update_initializer:
+            covs_layer.diagonals_initializer = WeightInitializer(diagonals)
+
+    else:
+        flattened_cholesky_factors = covs_layer.bijector.inverse(covariances)
+        layer_weights = covs_layer.flattened_cholesky_factors
+        layer_weights.assign(flattened_cholesky_factors)
+
+        if update_initializer:
+            covs_layer.flattened_cholesky_factors_initializer = WeightInitializer(
+                flattened_cholesky_factors
+            )
 
 
 def set_means_regularizer(model, training_dataset, layer_name="means"):
@@ -275,13 +325,20 @@ def set_means_regularizer(model, training_dataset, layer_name="means"):
     means_layer.regularizer = regularizers.MultivariateNormal(mu, sigma, n_batches)
 
 
-def set_covariances_regularizer(model, training_dataset, layer_name="covs"):
+def set_covariances_regularizer(
+    model, training_dataset, diagonal=False, layer_name="covs"
+):
     n_batches = dtf.get_n_batches(training_dataset)
     n_channels = dtf.get_n_channels(training_dataset)
     range_ = dtf.get_range(training_dataset)
 
-    nu = n_channels - 1 + 0.1
-    psi = np.diag(range_)
-
     covs_layer = model.get_layer(layer_name)
-    covs_layer.regularizer = regularizers.InverseWishart(nu, psi, n_batches)
+    if diagonal:
+        mu = np.zeros([n_channels], dtype=np.float32)
+        sigma = np.sqrt(np.log(2 * range_))
+        covs_layer.regularizer = regularizers.LogNormal(mu, sigma, n_batches)
+
+    else:
+        nu = n_channels - 1 + 0.1
+        psi = np.diag(range_)
+        covs_layer.regularizer = regularizers.InverseWishart(nu, psi, n_batches)
