@@ -1,4 +1,4 @@
-"""Subject Embedding Dynamic Network Modes (Se-DyNeMo).
+"""Subject Embedding Dynamic Network Modes (SE-DyNeMo).
 """
 
 from dataclasses import dataclass
@@ -18,7 +18,7 @@ from osl_dynamics.models.inf_mod_base import (
 from osl_dynamics.inference.layers import (
     InferenceRNNLayer,
     LogLikelihoodLossLayer,
-    MeanVectorsLayer,
+    VectorsLayer,
     CovarianceMatricesLayer,
     ModelRNNLayer,
     NormalizationLayer,
@@ -34,7 +34,7 @@ from osl_dynamics.inference.layers import (
     ZeroLayer,
     InverseCholeskyLayer,
     SubjectMapKLDivergenceLayer,
-    MLPLayer,
+    MultiLayerPerceptronLayer,
 )
 
 
@@ -255,10 +255,11 @@ class Config(BaseModelConfig, VariationalInferenceModelConfig):
 
 
 class Model(VariationalInferenceModelBase):
-    """Subject Embedded RNN Inference/model network and Gaussian observatons (SERIGO).
+    """SE-DyNeMo model class.
+
     Parameters
     ----------
-    config : dynemo.models.serigo.Config
+    config : dynemo.models.sedynemo.Config
     """
 
     config_type = Config
@@ -273,6 +274,7 @@ class Model(VariationalInferenceModelBase):
 
     def get_group_means_covariances(self):
         """Get the group means and covariances of each mode
+
         Returns
         -------
         means : np.ndarray
@@ -288,10 +290,11 @@ class Model(VariationalInferenceModelBase):
 
     def get_subject_embeddings(self):
         """Get the subject embedding vectors
+
         Returns
         -------
         subject_embeddings : np.ndarray
-            Embedding vectors for subjects. Shape is (n_subjects, subject_embedding_dim)
+            Embedding vectors for subjects. Shape is (n_subjects, subject_embedding_dim).
         """
         return sedynemo_obs.get_subject_embeddings(self.model)
 
@@ -309,6 +312,7 @@ class Model(VariationalInferenceModelBase):
 
     def get_concatenated_embeddings(self):
         """Get the concatenated embedding vectors of deviations.
+
         Returns
         -------
         means_embedding : np.ndarray
@@ -322,13 +326,14 @@ class Model(VariationalInferenceModelBase):
 
     def get_subject_dev(self):
         """Get the subject specific deviations of means and covs from the group.
+
         Returns
         -------
         means_dev : np.ndarray
             Deviation of means from the group. Shape is (n_subjects, n_modes, n_channels).
         covs_dev : np.ndarray
             Deviation of Cholesky factor of covs from the group.
-            Shaoe is (n_subjects, n_modes, n_channels * (n_channels + 1) // 2).
+            Shape is (n_subjects, n_modes, n_channels * (n_channels + 1) // 2).
         """
         return sedynemo_obs.get_subject_dev(self.model)
 
@@ -340,7 +345,8 @@ class Model(VariationalInferenceModelBase):
         subject_means : np.ndarray
             Mode means for each subject. Shape is (n_subjects, n_modes, n_channels).
         subject_covs : np.ndarray
-            Mode covariances for each subject. Shape is (n_subjects, n_modes, n_channels, n_channels).
+            Mode covariances for each subject.
+            Shape is (n_subjects, n_modes, n_channels, n_channels).
         """
         return sedynemo_obs.get_subject_means_covariances(self.model)
 
@@ -394,7 +400,10 @@ class Model(VariationalInferenceModelBase):
             the model?
         """
         dynemo_obs.set_covariances(
-            self.model, group_covariances, update_initializer, layer_name="group_covs"
+            model=self.model,
+            covariances=group_covariances,
+            update_initializer=update_initializer,
+            layer_name="group_covs",
         )
 
     def set_observation_model_parameters(
@@ -429,9 +438,13 @@ class Model(VariationalInferenceModelBase):
     def random_subject_initialization(
         self, training_data, n_epochs, n_subjects, n_kl_annealing_epochs=None, **kwargs
     ):
-        """random subject initialisation not compatible with SE-DyNeMo."""
+        """Random subject initialisation.
+
+        Currently not implemented in osl-dynamics. This method will raise
+        an Attribute error.
+        """
         raise AttributeError(
-            " 'Model' object has no attribute 'random_subject_initialization'."
+            "'Model' object has no attribute 'random_subject_initialization'."
         )
 
 
@@ -499,7 +512,7 @@ def _model_structure(config):
         config.n_subjects, config.subject_embedding_dim, name="subject_embeddings"
     )
 
-    group_means_layer = MeanVectorsLayer(
+    group_means_layer = VectorsLayer(
         config.n_modes,
         config.n_channels,
         config.learn_means,
@@ -538,7 +551,7 @@ def _model_structure(config):
     )
 
     # Inference part of the deviation
-    means_dev_inf_input_layer = MLPLayer(
+    means_dev_inf_input_layer = MultiLayerPerceptronLayer(
         config.dev_inf_n_layers,
         config.dev_inf_n_units,
         config.dev_inf_normalization,
@@ -560,7 +573,7 @@ def _model_structure(config):
             name="means_dev",
         )
 
-    covs_dev_inf_input_layer = MLPLayer(
+    covs_dev_inf_input_layer = MultiLayerPerceptronLayer(
         config.dev_inf_n_layers,
         config.dev_inf_n_units,
         config.dev_inf_normalization,
@@ -676,7 +689,7 @@ def _model_structure(config):
     kl_div = kl_div_layer([inf_mu, inf_sigma, mod_mu, mod_sigma])
 
     # Prior part of deviations
-    means_dev_mod_input_layer = MLPLayer(
+    means_dev_mod_input_layer = MultiLayerPerceptronLayer(
         config.dev_inf_n_layers,
         config.dev_inf_n_units,
         config.dev_inf_normalization,
@@ -687,7 +700,7 @@ def _model_structure(config):
     means_dev_mod_sigma_layer = layers.Dense(
         config.n_channels, activation="softplus", name="means_dev_mod_sigma"
     )
-    covs_dev_mod_input_layer = MLPLayer(
+    covs_dev_mod_input_layer = MultiLayerPerceptronLayer(
         config.dev_inf_n_layers,
         config.dev_inf_n_units,
         config.dev_inf_normalization,
