@@ -1274,8 +1274,8 @@ def plot_matrices(
             axis.remove()
             continue
         if group_color_scale:
-            v_min = matrix.min()
-            v_max = matrix.max()
+            v_min = np.nanmin(matrix)
+            v_max = np.nanmax(matrix)
             if log_norm:
                 im = axis.matshow(
                     grid,
@@ -1290,6 +1290,10 @@ def plot_matrices(
             else:
                 im = axis.matshow(grid, cmap=cmap)
         axis.set_title(title)
+        if grid.shape[0] > 30:
+            # Don't label the ticks if there's too many
+            axis.set_xticklabels([])
+            axis.set_yticklabels([])
 
     if group_color_scale:
         fig.subplots_adjust(right=0.8)
@@ -1561,9 +1565,9 @@ def topoplot(
 
 def plot_brain_surface(
     values,
-    filename,
     mask_file,
     parcellation_file,
+    filename=None,
     subtract_mean=False,
     mean_weights=None,
     **plot_kwargs,
@@ -1576,13 +1580,14 @@ def plot_brain_surface(
         Data to plot. Can be of shape: (n_maps, n_channels) or (n_channels,).
         A (..., n_channels, n_channels) array can also be passed.
         Warning: this function cannot be used if n_maps is equal to n_channels.
-    filename : str
-        Output filename. If extension is .nii.gz the power map is saved as a
-        NIFTI file. Or if the extension is png/svg/pdf, it is saved as images.
     mask_file : str
         Mask file used to preprocess the training data.
     parcellation_file : str
         Parcellation file used to parcelate the training data.
+    filename : str
+        Output filename. If extension is .nii.gz the power map is saved as a
+        NIFTI file. Or if the extension is png/svg/pdf, it is saved as images.
+        Optional, if None is passed then the image is shown on screen.
     subtract_mean : bool
         Should we subtract the mean power across modes?
     mean_weights: np.ndarray
@@ -1769,7 +1774,7 @@ def plot_mode_lifetimes(
     ax : matplotlib.pyplot.axis.
         Matplotlib axis object(s).
     """
-    from osl_dynamics.inference import modes
+    from osl_dynamics.analysis import modes
 
     n_plots = mode_time_course.shape[1]
     short, long, empty = rough_square_axes(n_plots)
